﻿using Hotline.Share.Enums.FlowEngine;
using SqlSugar;

namespace Hotline.FlowEngine.Workflows;

public class WorkflowStep : StepBasicEntity
{
    public string WorkflowId { get; set; }

    [SugarColumn(ColumnDataType = "varchar(4000)", IsJson = true)]
    public List<NextStep> NextSteps { get; set; }

    /// <summary>
    /// 被指派办理对象（依据不同指派方式可能为：depCode或userId）,该字段subStep才会存在，stepBox不存在
    /// </summary>
    public string HandlerId { get; set; }

    /// <summary>
    /// 办理人部门code
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? OrgCode { get; set; }

    [SugarColumn(IsNullable = true)]
    public string? OrgName { get; set; }

    /// <summary>
    /// 办理人
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? UserId { get; set; }

    [SugarColumn(IsNullable = true)]
    public string? UserName { get; set; }

    /// <summary>
    /// 办理完成时间
    /// </summary>
    public DateTime? CompleteTime { get; set; }

    /// <summary>
    /// 接办人
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? AcceptUserId { get; set; }

    [SugarColumn(IsNullable = true)]
    public string? AcceptUserName { get; set; }

    /// <summary>
    /// 接办时间
    /// </summary>
    public DateTime? AcceptTime { get; set; }

    /// <summary>
    /// 上一级节点Id(stepBox此字段为上级stepBoxId,step为上级stepId)
    /// </summary>
    public string PreviousId { get; set; }

    /// <summary>
    /// 主办
    /// </summary>
    public bool IsMain { get; set; }

    public EWorkflowStepStatus Status { get; set; } = EWorkflowStepStatus.Assigned;

    #region 审批参数

    /// <summary>
    /// (下一节点办理人)根据审批者类型不同，此字段为不同内容
    /// <example>
    /// 部门等级/分类为：orgCodes, 角色为：userIds
    /// </example>
    /// </summary>
    [SugarColumn(ColumnDataType = "varchar(2000)", IsJson = true)]
    public List<string> NextHandlers { get; set; }

    /// <summary>
    /// 下一节点主办，(NextHandlers其中一个, 如果不是会签则只有一个)
    /// </summary>
    public string NextMainHandler { get; set; }

    /// <summary>
    /// 下一节点code
    /// </summary>
    public string NextStepCode { get; set; }

    /// <summary>
    /// 是否短信通知
    /// </summary>
    public bool AcceptSms { get; set; }

    /// <summary>
    /// 办理意见
    /// </summary>
    [SugarColumn(Length = 2000)]
    public string Opinion { get; set; }

    /// <summary>
    /// 附件
    /// </summary>
    [SugarColumn(ColumnDataType = "varchar(1000)", IsJson = true)]
    public List<string> Additions { get; set; }

    #endregion

    /// <summary>
    /// stepBox此字段无效，记录stepBox与其下subSteps关系
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? ParentId { get; set; }

    [SugarColumn(IsIgnore = true)]
    public List<WorkflowStep> Steps { get; set; }

    /// <summary>
    /// 接办
    /// </summary>
    /// <param name="userId"></param>
    /// <param name="userName"></param>
    public void Accept(string userId, string userName)
    {
        AcceptUserId = userId;
        AcceptUserName = userName;
        AcceptTime = DateTime.Now;
        Status = EWorkflowStepStatus.Accepted;
    }

    /// <summary>
    /// 办理完成
    /// </summary>
    /// <param name="userId"></param>
    /// <param name="userName"></param>
    /// <param name="orgCode"></param>
    /// <param name="orgName"></param>
    public void Complete(string userId, string userName, string orgCode, string orgName, string nextStepCode)
    {
        UserId = userId;
        UserName = userName;
        OrgCode = orgCode;
        OrgName = orgName;
        CompleteTime = DateTime.Now;
        Status = EWorkflowStepStatus.Completed;

        NextSteps.First(d => d.Code == nextStepCode).Selected = true;
    }
}